# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['DomainArgs', 'Domain']

@pulumi.input_type
class DomainArgs:
    def __init__(__self__, *,
                 arch: Optional[pulumi.Input[str]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 boot_devices: Optional[pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]]] = None,
                 cloudinit: Optional[pulumi.Input[str]] = None,
                 cmdlines: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]] = None,
                 consoles: Optional[pulumi.Input[Sequence[pulumi.Input['DomainConsoleArgs']]]] = None,
                 coreos_ignition: Optional[pulumi.Input[str]] = None,
                 cpu: Optional[pulumi.Input['DomainCpuArgs']] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 disks: Optional[pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]]] = None,
                 emulator: Optional[pulumi.Input[str]] = None,
                 filesystems: Optional[pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]]] = None,
                 firmware: Optional[pulumi.Input[str]] = None,
                 fw_cfg_name: Optional[pulumi.Input[str]] = None,
                 graphics: Optional[pulumi.Input['DomainGraphicsArgs']] = None,
                 initrd: Optional[pulumi.Input[str]] = None,
                 kernel: Optional[pulumi.Input[str]] = None,
                 machine: Optional[pulumi.Input[str]] = None,
                 memory: Optional[pulumi.Input[int]] = None,
                 metadata: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]]] = None,
                 nvram: Optional[pulumi.Input['DomainNvramArgs']] = None,
                 qemu_agent: Optional[pulumi.Input[bool]] = None,
                 running: Optional[pulumi.Input[bool]] = None,
                 tpm: Optional[pulumi.Input['DomainTpmArgs']] = None,
                 vcpu: Optional[pulumi.Input[int]] = None,
                 video: Optional[pulumi.Input['DomainVideoArgs']] = None,
                 xml: Optional[pulumi.Input['DomainXmlArgs']] = None):
        """
        The set of arguments for constructing a Domain resource.
        :param pulumi.Input[str] arch: The architecture for the VM (probably x86_64 or i686),
               you normally won't need to set this unless you are building a special VM
        :param pulumi.Input[bool] autostart: Set to `true` to start the domain on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]] boot_devices: A list of devices (dev) which defines boot order. Example
               below.
        :param pulumi.Input[str] cloudinit: The `CloudInitDisk` disk that has to be used by
               the domain. This is going to be attached as a CDROM ISO. Changing the
               cloud-init won't cause the domain to be recreated, however the change will
               have effect on the next reboot.
        :param pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]] cmdlines: Arguments to the kernel
        :param pulumi.Input[str] coreos_ignition: The
               [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
               that is to be used by the CoreOS domain.
        :param pulumi.Input['DomainCpuArgs'] cpu: Configures CPU mode. See below for more
               details.
        :param pulumi.Input[str] description: The description for domain.
               Changing this forces a new resource to be created.
               This data is not used by libvirt in any way, it can contain any information the user wants.
        :param pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]] disks: An array of one or more disks to attach to the domain. The
               `disk` object structure is documented below.
        :param pulumi.Input[str] emulator: The path of the emulator to use
        :param pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]] filesystems: An array of one or more host filesystems to attach to
               the domain. The `filesystem` object structure is documented
               below.
        :param pulumi.Input[str] firmware: The UEFI rom images for exercising UEFI secure boot in a qemu
               environment. Users should usually specify one of the standard _Open Virtual Machine
               Firmware_ (_OVMF_) images available for their distributions. The file will be opened
               read-only.
        :param pulumi.Input[str] fw_cfg_name: The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        :param pulumi.Input[str] initrd: The path of the initrd to boot.
        :param pulumi.Input[str] kernel: The path of the kernel to boot
        :param pulumi.Input[str] machine: The machine type,
               you normally won't need to set this unless you are running on a platform that
               defaults to the wrong machine type for your template
        :param pulumi.Input[int] memory: The amount of memory in MiB. If not specified the domain
               will be created with 512 MiB of memory be used.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]] network_interfaces: An array of one or more network interfaces to
               attach to the domain. The `network_interface` object structure is documented
               below.
        :param pulumi.Input['DomainNvramArgs'] nvram: this block allows specifying the following attributes related to the _nvram_:
        :param pulumi.Input[bool] qemu_agent: By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        :param pulumi.Input[bool] running: Use `false` to turn off the instance. If not specified,
               true is assumed and the instance, if stopped, will be started at next apply.
        :param pulumi.Input['DomainTpmArgs'] tpm: TPM device to attach to the domain. The `tpm` object structure is documented below.
        :param pulumi.Input[int] vcpu: The amount of virtual CPUs. If not specified, a single CPU
               will be created.
        """
        if arch is not None:
            pulumi.set(__self__, "arch", arch)
        if autostart is not None:
            pulumi.set(__self__, "autostart", autostart)
        if boot_devices is not None:
            pulumi.set(__self__, "boot_devices", boot_devices)
        if cloudinit is not None:
            pulumi.set(__self__, "cloudinit", cloudinit)
        if cmdlines is not None:
            pulumi.set(__self__, "cmdlines", cmdlines)
        if consoles is not None:
            pulumi.set(__self__, "consoles", consoles)
        if coreos_ignition is not None:
            pulumi.set(__self__, "coreos_ignition", coreos_ignition)
        if cpu is not None:
            pulumi.set(__self__, "cpu", cpu)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if disks is not None:
            pulumi.set(__self__, "disks", disks)
        if emulator is not None:
            pulumi.set(__self__, "emulator", emulator)
        if filesystems is not None:
            pulumi.set(__self__, "filesystems", filesystems)
        if firmware is not None:
            pulumi.set(__self__, "firmware", firmware)
        if fw_cfg_name is not None:
            pulumi.set(__self__, "fw_cfg_name", fw_cfg_name)
        if graphics is not None:
            pulumi.set(__self__, "graphics", graphics)
        if initrd is not None:
            pulumi.set(__self__, "initrd", initrd)
        if kernel is not None:
            pulumi.set(__self__, "kernel", kernel)
        if machine is not None:
            pulumi.set(__self__, "machine", machine)
        if memory is not None:
            pulumi.set(__self__, "memory", memory)
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if network_interfaces is not None:
            pulumi.set(__self__, "network_interfaces", network_interfaces)
        if nvram is not None:
            pulumi.set(__self__, "nvram", nvram)
        if qemu_agent is not None:
            pulumi.set(__self__, "qemu_agent", qemu_agent)
        if running is not None:
            pulumi.set(__self__, "running", running)
        if tpm is not None:
            pulumi.set(__self__, "tpm", tpm)
        if vcpu is not None:
            pulumi.set(__self__, "vcpu", vcpu)
        if video is not None:
            pulumi.set(__self__, "video", video)
        if xml is not None:
            pulumi.set(__self__, "xml", xml)

    @property
    @pulumi.getter
    def arch(self) -> Optional[pulumi.Input[str]]:
        """
        The architecture for the VM (probably x86_64 or i686),
        you normally won't need to set this unless you are building a special VM
        """
        return pulumi.get(self, "arch")

    @arch.setter
    def arch(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "arch", value)

    @property
    @pulumi.getter
    def autostart(self) -> Optional[pulumi.Input[bool]]:
        """
        Set to `true` to start the domain on host boot up.
        If not specified `false` is assumed.
        """
        return pulumi.get(self, "autostart")

    @autostart.setter
    def autostart(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "autostart", value)

    @property
    @pulumi.getter(name="bootDevices")
    def boot_devices(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]]]:
        """
        A list of devices (dev) which defines boot order. Example
        below.
        """
        return pulumi.get(self, "boot_devices")

    @boot_devices.setter
    def boot_devices(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]]]):
        pulumi.set(self, "boot_devices", value)

    @property
    @pulumi.getter
    def cloudinit(self) -> Optional[pulumi.Input[str]]:
        """
        The `CloudInitDisk` disk that has to be used by
        the domain. This is going to be attached as a CDROM ISO. Changing the
        cloud-init won't cause the domain to be recreated, however the change will
        have effect on the next reboot.
        """
        return pulumi.get(self, "cloudinit")

    @cloudinit.setter
    def cloudinit(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cloudinit", value)

    @property
    @pulumi.getter
    def cmdlines(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]]:
        """
        Arguments to the kernel
        """
        return pulumi.get(self, "cmdlines")

    @cmdlines.setter
    def cmdlines(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]]):
        pulumi.set(self, "cmdlines", value)

    @property
    @pulumi.getter
    def consoles(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainConsoleArgs']]]]:
        return pulumi.get(self, "consoles")

    @consoles.setter
    def consoles(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainConsoleArgs']]]]):
        pulumi.set(self, "consoles", value)

    @property
    @pulumi.getter(name="coreosIgnition")
    def coreos_ignition(self) -> Optional[pulumi.Input[str]]:
        """
        The
        [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
        that is to be used by the CoreOS domain.
        """
        return pulumi.get(self, "coreos_ignition")

    @coreos_ignition.setter
    def coreos_ignition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "coreos_ignition", value)

    @property
    @pulumi.getter
    def cpu(self) -> Optional[pulumi.Input['DomainCpuArgs']]:
        """
        Configures CPU mode. See below for more
        details.
        """
        return pulumi.get(self, "cpu")

    @cpu.setter
    def cpu(self, value: Optional[pulumi.Input['DomainCpuArgs']]):
        pulumi.set(self, "cpu", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description for domain.
        Changing this forces a new resource to be created.
        This data is not used by libvirt in any way, it can contain any information the user wants.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def disks(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]]]:
        """
        An array of one or more disks to attach to the domain. The
        `disk` object structure is documented below.
        """
        return pulumi.get(self, "disks")

    @disks.setter
    def disks(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]]]):
        pulumi.set(self, "disks", value)

    @property
    @pulumi.getter
    def emulator(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the emulator to use
        """
        return pulumi.get(self, "emulator")

    @emulator.setter
    def emulator(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "emulator", value)

    @property
    @pulumi.getter
    def filesystems(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]]]:
        """
        An array of one or more host filesystems to attach to
        the domain. The `filesystem` object structure is documented
        below.
        """
        return pulumi.get(self, "filesystems")

    @filesystems.setter
    def filesystems(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]]]):
        pulumi.set(self, "filesystems", value)

    @property
    @pulumi.getter
    def firmware(self) -> Optional[pulumi.Input[str]]:
        """
        The UEFI rom images for exercising UEFI secure boot in a qemu
        environment. Users should usually specify one of the standard _Open Virtual Machine
        Firmware_ (_OVMF_) images available for their distributions. The file will be opened
        read-only.
        """
        return pulumi.get(self, "firmware")

    @firmware.setter
    def firmware(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "firmware", value)

    @property
    @pulumi.getter(name="fwCfgName")
    def fw_cfg_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        """
        return pulumi.get(self, "fw_cfg_name")

    @fw_cfg_name.setter
    def fw_cfg_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fw_cfg_name", value)

    @property
    @pulumi.getter
    def graphics(self) -> Optional[pulumi.Input['DomainGraphicsArgs']]:
        return pulumi.get(self, "graphics")

    @graphics.setter
    def graphics(self, value: Optional[pulumi.Input['DomainGraphicsArgs']]):
        pulumi.set(self, "graphics", value)

    @property
    @pulumi.getter
    def initrd(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the initrd to boot.
        """
        return pulumi.get(self, "initrd")

    @initrd.setter
    def initrd(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "initrd", value)

    @property
    @pulumi.getter
    def kernel(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the kernel to boot
        """
        return pulumi.get(self, "kernel")

    @kernel.setter
    def kernel(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kernel", value)

    @property
    @pulumi.getter
    def machine(self) -> Optional[pulumi.Input[str]]:
        """
        The machine type,
        you normally won't need to set this unless you are running on a platform that
        defaults to the wrong machine type for your template
        """
        return pulumi.get(self, "machine")

    @machine.setter
    def machine(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "machine", value)

    @property
    @pulumi.getter
    def memory(self) -> Optional[pulumi.Input[int]]:
        """
        The amount of memory in MiB. If not specified the domain
        will be created with 512 MiB of memory be used.
        """
        return pulumi.get(self, "memory")

    @memory.setter
    def memory(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "memory", value)

    @property
    @pulumi.getter
    def metadata(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "metadata")

    @metadata.setter
    def metadata(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "metadata", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="networkInterfaces")
    def network_interfaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]]]:
        """
        An array of one or more network interfaces to
        attach to the domain. The `network_interface` object structure is documented
        below.
        """
        return pulumi.get(self, "network_interfaces")

    @network_interfaces.setter
    def network_interfaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]]]):
        pulumi.set(self, "network_interfaces", value)

    @property
    @pulumi.getter
    def nvram(self) -> Optional[pulumi.Input['DomainNvramArgs']]:
        """
        this block allows specifying the following attributes related to the _nvram_:
        """
        return pulumi.get(self, "nvram")

    @nvram.setter
    def nvram(self, value: Optional[pulumi.Input['DomainNvramArgs']]):
        pulumi.set(self, "nvram", value)

    @property
    @pulumi.getter(name="qemuAgent")
    def qemu_agent(self) -> Optional[pulumi.Input[bool]]:
        """
        By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        """
        return pulumi.get(self, "qemu_agent")

    @qemu_agent.setter
    def qemu_agent(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "qemu_agent", value)

    @property
    @pulumi.getter
    def running(self) -> Optional[pulumi.Input[bool]]:
        """
        Use `false` to turn off the instance. If not specified,
        true is assumed and the instance, if stopped, will be started at next apply.
        """
        return pulumi.get(self, "running")

    @running.setter
    def running(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "running", value)

    @property
    @pulumi.getter
    def tpm(self) -> Optional[pulumi.Input['DomainTpmArgs']]:
        """
        TPM device to attach to the domain. The `tpm` object structure is documented below.
        """
        return pulumi.get(self, "tpm")

    @tpm.setter
    def tpm(self, value: Optional[pulumi.Input['DomainTpmArgs']]):
        pulumi.set(self, "tpm", value)

    @property
    @pulumi.getter
    def vcpu(self) -> Optional[pulumi.Input[int]]:
        """
        The amount of virtual CPUs. If not specified, a single CPU
        will be created.
        """
        return pulumi.get(self, "vcpu")

    @vcpu.setter
    def vcpu(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vcpu", value)

    @property
    @pulumi.getter
    def video(self) -> Optional[pulumi.Input['DomainVideoArgs']]:
        return pulumi.get(self, "video")

    @video.setter
    def video(self, value: Optional[pulumi.Input['DomainVideoArgs']]):
        pulumi.set(self, "video", value)

    @property
    @pulumi.getter
    def xml(self) -> Optional[pulumi.Input['DomainXmlArgs']]:
        return pulumi.get(self, "xml")

    @xml.setter
    def xml(self, value: Optional[pulumi.Input['DomainXmlArgs']]):
        pulumi.set(self, "xml", value)


@pulumi.input_type
class _DomainState:
    def __init__(__self__, *,
                 arch: Optional[pulumi.Input[str]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 boot_devices: Optional[pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]]] = None,
                 cloudinit: Optional[pulumi.Input[str]] = None,
                 cmdlines: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]] = None,
                 consoles: Optional[pulumi.Input[Sequence[pulumi.Input['DomainConsoleArgs']]]] = None,
                 coreos_ignition: Optional[pulumi.Input[str]] = None,
                 cpu: Optional[pulumi.Input['DomainCpuArgs']] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 disks: Optional[pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]]] = None,
                 emulator: Optional[pulumi.Input[str]] = None,
                 filesystems: Optional[pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]]] = None,
                 firmware: Optional[pulumi.Input[str]] = None,
                 fw_cfg_name: Optional[pulumi.Input[str]] = None,
                 graphics: Optional[pulumi.Input['DomainGraphicsArgs']] = None,
                 initrd: Optional[pulumi.Input[str]] = None,
                 kernel: Optional[pulumi.Input[str]] = None,
                 machine: Optional[pulumi.Input[str]] = None,
                 memory: Optional[pulumi.Input[int]] = None,
                 metadata: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]]] = None,
                 nvram: Optional[pulumi.Input['DomainNvramArgs']] = None,
                 qemu_agent: Optional[pulumi.Input[bool]] = None,
                 running: Optional[pulumi.Input[bool]] = None,
                 tpm: Optional[pulumi.Input['DomainTpmArgs']] = None,
                 vcpu: Optional[pulumi.Input[int]] = None,
                 video: Optional[pulumi.Input['DomainVideoArgs']] = None,
                 xml: Optional[pulumi.Input['DomainXmlArgs']] = None):
        """
        Input properties used for looking up and filtering Domain resources.
        :param pulumi.Input[str] arch: The architecture for the VM (probably x86_64 or i686),
               you normally won't need to set this unless you are building a special VM
        :param pulumi.Input[bool] autostart: Set to `true` to start the domain on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]] boot_devices: A list of devices (dev) which defines boot order. Example
               below.
        :param pulumi.Input[str] cloudinit: The `CloudInitDisk` disk that has to be used by
               the domain. This is going to be attached as a CDROM ISO. Changing the
               cloud-init won't cause the domain to be recreated, however the change will
               have effect on the next reboot.
        :param pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]] cmdlines: Arguments to the kernel
        :param pulumi.Input[str] coreos_ignition: The
               [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
               that is to be used by the CoreOS domain.
        :param pulumi.Input['DomainCpuArgs'] cpu: Configures CPU mode. See below for more
               details.
        :param pulumi.Input[str] description: The description for domain.
               Changing this forces a new resource to be created.
               This data is not used by libvirt in any way, it can contain any information the user wants.
        :param pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]] disks: An array of one or more disks to attach to the domain. The
               `disk` object structure is documented below.
        :param pulumi.Input[str] emulator: The path of the emulator to use
        :param pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]] filesystems: An array of one or more host filesystems to attach to
               the domain. The `filesystem` object structure is documented
               below.
        :param pulumi.Input[str] firmware: The UEFI rom images for exercising UEFI secure boot in a qemu
               environment. Users should usually specify one of the standard _Open Virtual Machine
               Firmware_ (_OVMF_) images available for their distributions. The file will be opened
               read-only.
        :param pulumi.Input[str] fw_cfg_name: The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        :param pulumi.Input[str] initrd: The path of the initrd to boot.
        :param pulumi.Input[str] kernel: The path of the kernel to boot
        :param pulumi.Input[str] machine: The machine type,
               you normally won't need to set this unless you are running on a platform that
               defaults to the wrong machine type for your template
        :param pulumi.Input[int] memory: The amount of memory in MiB. If not specified the domain
               will be created with 512 MiB of memory be used.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]] network_interfaces: An array of one or more network interfaces to
               attach to the domain. The `network_interface` object structure is documented
               below.
        :param pulumi.Input['DomainNvramArgs'] nvram: this block allows specifying the following attributes related to the _nvram_:
        :param pulumi.Input[bool] qemu_agent: By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        :param pulumi.Input[bool] running: Use `false` to turn off the instance. If not specified,
               true is assumed and the instance, if stopped, will be started at next apply.
        :param pulumi.Input['DomainTpmArgs'] tpm: TPM device to attach to the domain. The `tpm` object structure is documented below.
        :param pulumi.Input[int] vcpu: The amount of virtual CPUs. If not specified, a single CPU
               will be created.
        """
        if arch is not None:
            pulumi.set(__self__, "arch", arch)
        if autostart is not None:
            pulumi.set(__self__, "autostart", autostart)
        if boot_devices is not None:
            pulumi.set(__self__, "boot_devices", boot_devices)
        if cloudinit is not None:
            pulumi.set(__self__, "cloudinit", cloudinit)
        if cmdlines is not None:
            pulumi.set(__self__, "cmdlines", cmdlines)
        if consoles is not None:
            pulumi.set(__self__, "consoles", consoles)
        if coreos_ignition is not None:
            pulumi.set(__self__, "coreos_ignition", coreos_ignition)
        if cpu is not None:
            pulumi.set(__self__, "cpu", cpu)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if disks is not None:
            pulumi.set(__self__, "disks", disks)
        if emulator is not None:
            pulumi.set(__self__, "emulator", emulator)
        if filesystems is not None:
            pulumi.set(__self__, "filesystems", filesystems)
        if firmware is not None:
            pulumi.set(__self__, "firmware", firmware)
        if fw_cfg_name is not None:
            pulumi.set(__self__, "fw_cfg_name", fw_cfg_name)
        if graphics is not None:
            pulumi.set(__self__, "graphics", graphics)
        if initrd is not None:
            pulumi.set(__self__, "initrd", initrd)
        if kernel is not None:
            pulumi.set(__self__, "kernel", kernel)
        if machine is not None:
            pulumi.set(__self__, "machine", machine)
        if memory is not None:
            pulumi.set(__self__, "memory", memory)
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if network_interfaces is not None:
            pulumi.set(__self__, "network_interfaces", network_interfaces)
        if nvram is not None:
            pulumi.set(__self__, "nvram", nvram)
        if qemu_agent is not None:
            pulumi.set(__self__, "qemu_agent", qemu_agent)
        if running is not None:
            pulumi.set(__self__, "running", running)
        if tpm is not None:
            pulumi.set(__self__, "tpm", tpm)
        if vcpu is not None:
            pulumi.set(__self__, "vcpu", vcpu)
        if video is not None:
            pulumi.set(__self__, "video", video)
        if xml is not None:
            pulumi.set(__self__, "xml", xml)

    @property
    @pulumi.getter
    def arch(self) -> Optional[pulumi.Input[str]]:
        """
        The architecture for the VM (probably x86_64 or i686),
        you normally won't need to set this unless you are building a special VM
        """
        return pulumi.get(self, "arch")

    @arch.setter
    def arch(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "arch", value)

    @property
    @pulumi.getter
    def autostart(self) -> Optional[pulumi.Input[bool]]:
        """
        Set to `true` to start the domain on host boot up.
        If not specified `false` is assumed.
        """
        return pulumi.get(self, "autostart")

    @autostart.setter
    def autostart(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "autostart", value)

    @property
    @pulumi.getter(name="bootDevices")
    def boot_devices(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]]]:
        """
        A list of devices (dev) which defines boot order. Example
        below.
        """
        return pulumi.get(self, "boot_devices")

    @boot_devices.setter
    def boot_devices(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainBootDeviceArgs']]]]):
        pulumi.set(self, "boot_devices", value)

    @property
    @pulumi.getter
    def cloudinit(self) -> Optional[pulumi.Input[str]]:
        """
        The `CloudInitDisk` disk that has to be used by
        the domain. This is going to be attached as a CDROM ISO. Changing the
        cloud-init won't cause the domain to be recreated, however the change will
        have effect on the next reboot.
        """
        return pulumi.get(self, "cloudinit")

    @cloudinit.setter
    def cloudinit(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cloudinit", value)

    @property
    @pulumi.getter
    def cmdlines(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]]:
        """
        Arguments to the kernel
        """
        return pulumi.get(self, "cmdlines")

    @cmdlines.setter
    def cmdlines(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]]):
        pulumi.set(self, "cmdlines", value)

    @property
    @pulumi.getter
    def consoles(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainConsoleArgs']]]]:
        return pulumi.get(self, "consoles")

    @consoles.setter
    def consoles(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainConsoleArgs']]]]):
        pulumi.set(self, "consoles", value)

    @property
    @pulumi.getter(name="coreosIgnition")
    def coreos_ignition(self) -> Optional[pulumi.Input[str]]:
        """
        The
        [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
        that is to be used by the CoreOS domain.
        """
        return pulumi.get(self, "coreos_ignition")

    @coreos_ignition.setter
    def coreos_ignition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "coreos_ignition", value)

    @property
    @pulumi.getter
    def cpu(self) -> Optional[pulumi.Input['DomainCpuArgs']]:
        """
        Configures CPU mode. See below for more
        details.
        """
        return pulumi.get(self, "cpu")

    @cpu.setter
    def cpu(self, value: Optional[pulumi.Input['DomainCpuArgs']]):
        pulumi.set(self, "cpu", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description for domain.
        Changing this forces a new resource to be created.
        This data is not used by libvirt in any way, it can contain any information the user wants.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def disks(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]]]:
        """
        An array of one or more disks to attach to the domain. The
        `disk` object structure is documented below.
        """
        return pulumi.get(self, "disks")

    @disks.setter
    def disks(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainDiskArgs']]]]):
        pulumi.set(self, "disks", value)

    @property
    @pulumi.getter
    def emulator(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the emulator to use
        """
        return pulumi.get(self, "emulator")

    @emulator.setter
    def emulator(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "emulator", value)

    @property
    @pulumi.getter
    def filesystems(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]]]:
        """
        An array of one or more host filesystems to attach to
        the domain. The `filesystem` object structure is documented
        below.
        """
        return pulumi.get(self, "filesystems")

    @filesystems.setter
    def filesystems(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainFilesystemArgs']]]]):
        pulumi.set(self, "filesystems", value)

    @property
    @pulumi.getter
    def firmware(self) -> Optional[pulumi.Input[str]]:
        """
        The UEFI rom images for exercising UEFI secure boot in a qemu
        environment. Users should usually specify one of the standard _Open Virtual Machine
        Firmware_ (_OVMF_) images available for their distributions. The file will be opened
        read-only.
        """
        return pulumi.get(self, "firmware")

    @firmware.setter
    def firmware(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "firmware", value)

    @property
    @pulumi.getter(name="fwCfgName")
    def fw_cfg_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        """
        return pulumi.get(self, "fw_cfg_name")

    @fw_cfg_name.setter
    def fw_cfg_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fw_cfg_name", value)

    @property
    @pulumi.getter
    def graphics(self) -> Optional[pulumi.Input['DomainGraphicsArgs']]:
        return pulumi.get(self, "graphics")

    @graphics.setter
    def graphics(self, value: Optional[pulumi.Input['DomainGraphicsArgs']]):
        pulumi.set(self, "graphics", value)

    @property
    @pulumi.getter
    def initrd(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the initrd to boot.
        """
        return pulumi.get(self, "initrd")

    @initrd.setter
    def initrd(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "initrd", value)

    @property
    @pulumi.getter
    def kernel(self) -> Optional[pulumi.Input[str]]:
        """
        The path of the kernel to boot
        """
        return pulumi.get(self, "kernel")

    @kernel.setter
    def kernel(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kernel", value)

    @property
    @pulumi.getter
    def machine(self) -> Optional[pulumi.Input[str]]:
        """
        The machine type,
        you normally won't need to set this unless you are running on a platform that
        defaults to the wrong machine type for your template
        """
        return pulumi.get(self, "machine")

    @machine.setter
    def machine(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "machine", value)

    @property
    @pulumi.getter
    def memory(self) -> Optional[pulumi.Input[int]]:
        """
        The amount of memory in MiB. If not specified the domain
        will be created with 512 MiB of memory be used.
        """
        return pulumi.get(self, "memory")

    @memory.setter
    def memory(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "memory", value)

    @property
    @pulumi.getter
    def metadata(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "metadata")

    @metadata.setter
    def metadata(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "metadata", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="networkInterfaces")
    def network_interfaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]]]:
        """
        An array of one or more network interfaces to
        attach to the domain. The `network_interface` object structure is documented
        below.
        """
        return pulumi.get(self, "network_interfaces")

    @network_interfaces.setter
    def network_interfaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DomainNetworkInterfaceArgs']]]]):
        pulumi.set(self, "network_interfaces", value)

    @property
    @pulumi.getter
    def nvram(self) -> Optional[pulumi.Input['DomainNvramArgs']]:
        """
        this block allows specifying the following attributes related to the _nvram_:
        """
        return pulumi.get(self, "nvram")

    @nvram.setter
    def nvram(self, value: Optional[pulumi.Input['DomainNvramArgs']]):
        pulumi.set(self, "nvram", value)

    @property
    @pulumi.getter(name="qemuAgent")
    def qemu_agent(self) -> Optional[pulumi.Input[bool]]:
        """
        By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        """
        return pulumi.get(self, "qemu_agent")

    @qemu_agent.setter
    def qemu_agent(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "qemu_agent", value)

    @property
    @pulumi.getter
    def running(self) -> Optional[pulumi.Input[bool]]:
        """
        Use `false` to turn off the instance. If not specified,
        true is assumed and the instance, if stopped, will be started at next apply.
        """
        return pulumi.get(self, "running")

    @running.setter
    def running(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "running", value)

    @property
    @pulumi.getter
    def tpm(self) -> Optional[pulumi.Input['DomainTpmArgs']]:
        """
        TPM device to attach to the domain. The `tpm` object structure is documented below.
        """
        return pulumi.get(self, "tpm")

    @tpm.setter
    def tpm(self, value: Optional[pulumi.Input['DomainTpmArgs']]):
        pulumi.set(self, "tpm", value)

    @property
    @pulumi.getter
    def vcpu(self) -> Optional[pulumi.Input[int]]:
        """
        The amount of virtual CPUs. If not specified, a single CPU
        will be created.
        """
        return pulumi.get(self, "vcpu")

    @vcpu.setter
    def vcpu(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vcpu", value)

    @property
    @pulumi.getter
    def video(self) -> Optional[pulumi.Input['DomainVideoArgs']]:
        return pulumi.get(self, "video")

    @video.setter
    def video(self, value: Optional[pulumi.Input['DomainVideoArgs']]):
        pulumi.set(self, "video", value)

    @property
    @pulumi.getter
    def xml(self) -> Optional[pulumi.Input['DomainXmlArgs']]:
        return pulumi.get(self, "xml")

    @xml.setter
    def xml(self, value: Optional[pulumi.Input['DomainXmlArgs']]):
        pulumi.set(self, "xml", value)


class Domain(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 arch: Optional[pulumi.Input[str]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 boot_devices: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainBootDeviceArgs']]]]] = None,
                 cloudinit: Optional[pulumi.Input[str]] = None,
                 cmdlines: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]] = None,
                 consoles: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainConsoleArgs']]]]] = None,
                 coreos_ignition: Optional[pulumi.Input[str]] = None,
                 cpu: Optional[pulumi.Input[pulumi.InputType['DomainCpuArgs']]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 disks: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainDiskArgs']]]]] = None,
                 emulator: Optional[pulumi.Input[str]] = None,
                 filesystems: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainFilesystemArgs']]]]] = None,
                 firmware: Optional[pulumi.Input[str]] = None,
                 fw_cfg_name: Optional[pulumi.Input[str]] = None,
                 graphics: Optional[pulumi.Input[pulumi.InputType['DomainGraphicsArgs']]] = None,
                 initrd: Optional[pulumi.Input[str]] = None,
                 kernel: Optional[pulumi.Input[str]] = None,
                 machine: Optional[pulumi.Input[str]] = None,
                 memory: Optional[pulumi.Input[int]] = None,
                 metadata: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainNetworkInterfaceArgs']]]]] = None,
                 nvram: Optional[pulumi.Input[pulumi.InputType['DomainNvramArgs']]] = None,
                 qemu_agent: Optional[pulumi.Input[bool]] = None,
                 running: Optional[pulumi.Input[bool]] = None,
                 tpm: Optional[pulumi.Input[pulumi.InputType['DomainTpmArgs']]] = None,
                 vcpu: Optional[pulumi.Input[int]] = None,
                 video: Optional[pulumi.Input[pulumi.InputType['DomainVideoArgs']]] = None,
                 xml: Optional[pulumi.Input[pulumi.InputType['DomainXmlArgs']]] = None,
                 __props__=None):
        """
        Manages a VM domain resource within libvirt. For more information see
        [the official documentation](https://libvirt.org/formatdomain.html).

        ## Example Usage

        ```python
        import pulumi
        import pulumi_libvirt as libvirt

        default = libvirt.Domain("default")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] arch: The architecture for the VM (probably x86_64 or i686),
               you normally won't need to set this unless you are building a special VM
        :param pulumi.Input[bool] autostart: Set to `true` to start the domain on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainBootDeviceArgs']]]] boot_devices: A list of devices (dev) which defines boot order. Example
               below.
        :param pulumi.Input[str] cloudinit: The `CloudInitDisk` disk that has to be used by
               the domain. This is going to be attached as a CDROM ISO. Changing the
               cloud-init won't cause the domain to be recreated, however the change will
               have effect on the next reboot.
        :param pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]] cmdlines: Arguments to the kernel
        :param pulumi.Input[str] coreos_ignition: The
               [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
               that is to be used by the CoreOS domain.
        :param pulumi.Input[pulumi.InputType['DomainCpuArgs']] cpu: Configures CPU mode. See below for more
               details.
        :param pulumi.Input[str] description: The description for domain.
               Changing this forces a new resource to be created.
               This data is not used by libvirt in any way, it can contain any information the user wants.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainDiskArgs']]]] disks: An array of one or more disks to attach to the domain. The
               `disk` object structure is documented below.
        :param pulumi.Input[str] emulator: The path of the emulator to use
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainFilesystemArgs']]]] filesystems: An array of one or more host filesystems to attach to
               the domain. The `filesystem` object structure is documented
               below.
        :param pulumi.Input[str] firmware: The UEFI rom images for exercising UEFI secure boot in a qemu
               environment. Users should usually specify one of the standard _Open Virtual Machine
               Firmware_ (_OVMF_) images available for their distributions. The file will be opened
               read-only.
        :param pulumi.Input[str] fw_cfg_name: The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        :param pulumi.Input[str] initrd: The path of the initrd to boot.
        :param pulumi.Input[str] kernel: The path of the kernel to boot
        :param pulumi.Input[str] machine: The machine type,
               you normally won't need to set this unless you are running on a platform that
               defaults to the wrong machine type for your template
        :param pulumi.Input[int] memory: The amount of memory in MiB. If not specified the domain
               will be created with 512 MiB of memory be used.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainNetworkInterfaceArgs']]]] network_interfaces: An array of one or more network interfaces to
               attach to the domain. The `network_interface` object structure is documented
               below.
        :param pulumi.Input[pulumi.InputType['DomainNvramArgs']] nvram: this block allows specifying the following attributes related to the _nvram_:
        :param pulumi.Input[bool] qemu_agent: By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        :param pulumi.Input[bool] running: Use `false` to turn off the instance. If not specified,
               true is assumed and the instance, if stopped, will be started at next apply.
        :param pulumi.Input[pulumi.InputType['DomainTpmArgs']] tpm: TPM device to attach to the domain. The `tpm` object structure is documented below.
        :param pulumi.Input[int] vcpu: The amount of virtual CPUs. If not specified, a single CPU
               will be created.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[DomainArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a VM domain resource within libvirt. For more information see
        [the official documentation](https://libvirt.org/formatdomain.html).

        ## Example Usage

        ```python
        import pulumi
        import pulumi_libvirt as libvirt

        default = libvirt.Domain("default")
        ```

        :param str resource_name: The name of the resource.
        :param DomainArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(DomainArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 arch: Optional[pulumi.Input[str]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 boot_devices: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainBootDeviceArgs']]]]] = None,
                 cloudinit: Optional[pulumi.Input[str]] = None,
                 cmdlines: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]] = None,
                 consoles: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainConsoleArgs']]]]] = None,
                 coreos_ignition: Optional[pulumi.Input[str]] = None,
                 cpu: Optional[pulumi.Input[pulumi.InputType['DomainCpuArgs']]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 disks: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainDiskArgs']]]]] = None,
                 emulator: Optional[pulumi.Input[str]] = None,
                 filesystems: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainFilesystemArgs']]]]] = None,
                 firmware: Optional[pulumi.Input[str]] = None,
                 fw_cfg_name: Optional[pulumi.Input[str]] = None,
                 graphics: Optional[pulumi.Input[pulumi.InputType['DomainGraphicsArgs']]] = None,
                 initrd: Optional[pulumi.Input[str]] = None,
                 kernel: Optional[pulumi.Input[str]] = None,
                 machine: Optional[pulumi.Input[str]] = None,
                 memory: Optional[pulumi.Input[int]] = None,
                 metadata: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainNetworkInterfaceArgs']]]]] = None,
                 nvram: Optional[pulumi.Input[pulumi.InputType['DomainNvramArgs']]] = None,
                 qemu_agent: Optional[pulumi.Input[bool]] = None,
                 running: Optional[pulumi.Input[bool]] = None,
                 tpm: Optional[pulumi.Input[pulumi.InputType['DomainTpmArgs']]] = None,
                 vcpu: Optional[pulumi.Input[int]] = None,
                 video: Optional[pulumi.Input[pulumi.InputType['DomainVideoArgs']]] = None,
                 xml: Optional[pulumi.Input[pulumi.InputType['DomainXmlArgs']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = DomainArgs.__new__(DomainArgs)

            __props__.__dict__["arch"] = arch
            __props__.__dict__["autostart"] = autostart
            __props__.__dict__["boot_devices"] = boot_devices
            __props__.__dict__["cloudinit"] = cloudinit
            __props__.__dict__["cmdlines"] = cmdlines
            __props__.__dict__["consoles"] = consoles
            __props__.__dict__["coreos_ignition"] = coreos_ignition
            __props__.__dict__["cpu"] = cpu
            __props__.__dict__["description"] = description
            __props__.__dict__["disks"] = disks
            __props__.__dict__["emulator"] = emulator
            __props__.__dict__["filesystems"] = filesystems
            __props__.__dict__["firmware"] = firmware
            __props__.__dict__["fw_cfg_name"] = fw_cfg_name
            __props__.__dict__["graphics"] = graphics
            __props__.__dict__["initrd"] = initrd
            __props__.__dict__["kernel"] = kernel
            __props__.__dict__["machine"] = machine
            __props__.__dict__["memory"] = memory
            __props__.__dict__["metadata"] = metadata
            __props__.__dict__["name"] = name
            __props__.__dict__["network_interfaces"] = network_interfaces
            __props__.__dict__["nvram"] = nvram
            __props__.__dict__["qemu_agent"] = qemu_agent
            __props__.__dict__["running"] = running
            __props__.__dict__["tpm"] = tpm
            __props__.__dict__["vcpu"] = vcpu
            __props__.__dict__["video"] = video
            __props__.__dict__["xml"] = xml
        super(Domain, __self__).__init__(
            'libvirt:index/domain:Domain',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            arch: Optional[pulumi.Input[str]] = None,
            autostart: Optional[pulumi.Input[bool]] = None,
            boot_devices: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainBootDeviceArgs']]]]] = None,
            cloudinit: Optional[pulumi.Input[str]] = None,
            cmdlines: Optional[pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]]] = None,
            consoles: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainConsoleArgs']]]]] = None,
            coreos_ignition: Optional[pulumi.Input[str]] = None,
            cpu: Optional[pulumi.Input[pulumi.InputType['DomainCpuArgs']]] = None,
            description: Optional[pulumi.Input[str]] = None,
            disks: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainDiskArgs']]]]] = None,
            emulator: Optional[pulumi.Input[str]] = None,
            filesystems: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainFilesystemArgs']]]]] = None,
            firmware: Optional[pulumi.Input[str]] = None,
            fw_cfg_name: Optional[pulumi.Input[str]] = None,
            graphics: Optional[pulumi.Input[pulumi.InputType['DomainGraphicsArgs']]] = None,
            initrd: Optional[pulumi.Input[str]] = None,
            kernel: Optional[pulumi.Input[str]] = None,
            machine: Optional[pulumi.Input[str]] = None,
            memory: Optional[pulumi.Input[int]] = None,
            metadata: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            network_interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainNetworkInterfaceArgs']]]]] = None,
            nvram: Optional[pulumi.Input[pulumi.InputType['DomainNvramArgs']]] = None,
            qemu_agent: Optional[pulumi.Input[bool]] = None,
            running: Optional[pulumi.Input[bool]] = None,
            tpm: Optional[pulumi.Input[pulumi.InputType['DomainTpmArgs']]] = None,
            vcpu: Optional[pulumi.Input[int]] = None,
            video: Optional[pulumi.Input[pulumi.InputType['DomainVideoArgs']]] = None,
            xml: Optional[pulumi.Input[pulumi.InputType['DomainXmlArgs']]] = None) -> 'Domain':
        """
        Get an existing Domain resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] arch: The architecture for the VM (probably x86_64 or i686),
               you normally won't need to set this unless you are building a special VM
        :param pulumi.Input[bool] autostart: Set to `true` to start the domain on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainBootDeviceArgs']]]] boot_devices: A list of devices (dev) which defines boot order. Example
               below.
        :param pulumi.Input[str] cloudinit: The `CloudInitDisk` disk that has to be used by
               the domain. This is going to be attached as a CDROM ISO. Changing the
               cloud-init won't cause the domain to be recreated, however the change will
               have effect on the next reboot.
        :param pulumi.Input[Sequence[pulumi.Input[Mapping[str, Any]]]] cmdlines: Arguments to the kernel
        :param pulumi.Input[str] coreos_ignition: The
               [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
               that is to be used by the CoreOS domain.
        :param pulumi.Input[pulumi.InputType['DomainCpuArgs']] cpu: Configures CPU mode. See below for more
               details.
        :param pulumi.Input[str] description: The description for domain.
               Changing this forces a new resource to be created.
               This data is not used by libvirt in any way, it can contain any information the user wants.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainDiskArgs']]]] disks: An array of one or more disks to attach to the domain. The
               `disk` object structure is documented below.
        :param pulumi.Input[str] emulator: The path of the emulator to use
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainFilesystemArgs']]]] filesystems: An array of one or more host filesystems to attach to
               the domain. The `filesystem` object structure is documented
               below.
        :param pulumi.Input[str] firmware: The UEFI rom images for exercising UEFI secure boot in a qemu
               environment. Users should usually specify one of the standard _Open Virtual Machine
               Firmware_ (_OVMF_) images available for their distributions. The file will be opened
               read-only.
        :param pulumi.Input[str] fw_cfg_name: The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        :param pulumi.Input[str] initrd: The path of the initrd to boot.
        :param pulumi.Input[str] kernel: The path of the kernel to boot
        :param pulumi.Input[str] machine: The machine type,
               you normally won't need to set this unless you are running on a platform that
               defaults to the wrong machine type for your template
        :param pulumi.Input[int] memory: The amount of memory in MiB. If not specified the domain
               will be created with 512 MiB of memory be used.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['DomainNetworkInterfaceArgs']]]] network_interfaces: An array of one or more network interfaces to
               attach to the domain. The `network_interface` object structure is documented
               below.
        :param pulumi.Input[pulumi.InputType['DomainNvramArgs']] nvram: this block allows specifying the following attributes related to the _nvram_:
        :param pulumi.Input[bool] qemu_agent: By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        :param pulumi.Input[bool] running: Use `false` to turn off the instance. If not specified,
               true is assumed and the instance, if stopped, will be started at next apply.
        :param pulumi.Input[pulumi.InputType['DomainTpmArgs']] tpm: TPM device to attach to the domain. The `tpm` object structure is documented below.
        :param pulumi.Input[int] vcpu: The amount of virtual CPUs. If not specified, a single CPU
               will be created.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _DomainState.__new__(_DomainState)

        __props__.__dict__["arch"] = arch
        __props__.__dict__["autostart"] = autostart
        __props__.__dict__["boot_devices"] = boot_devices
        __props__.__dict__["cloudinit"] = cloudinit
        __props__.__dict__["cmdlines"] = cmdlines
        __props__.__dict__["consoles"] = consoles
        __props__.__dict__["coreos_ignition"] = coreos_ignition
        __props__.__dict__["cpu"] = cpu
        __props__.__dict__["description"] = description
        __props__.__dict__["disks"] = disks
        __props__.__dict__["emulator"] = emulator
        __props__.__dict__["filesystems"] = filesystems
        __props__.__dict__["firmware"] = firmware
        __props__.__dict__["fw_cfg_name"] = fw_cfg_name
        __props__.__dict__["graphics"] = graphics
        __props__.__dict__["initrd"] = initrd
        __props__.__dict__["kernel"] = kernel
        __props__.__dict__["machine"] = machine
        __props__.__dict__["memory"] = memory
        __props__.__dict__["metadata"] = metadata
        __props__.__dict__["name"] = name
        __props__.__dict__["network_interfaces"] = network_interfaces
        __props__.__dict__["nvram"] = nvram
        __props__.__dict__["qemu_agent"] = qemu_agent
        __props__.__dict__["running"] = running
        __props__.__dict__["tpm"] = tpm
        __props__.__dict__["vcpu"] = vcpu
        __props__.__dict__["video"] = video
        __props__.__dict__["xml"] = xml
        return Domain(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def arch(self) -> pulumi.Output[str]:
        """
        The architecture for the VM (probably x86_64 or i686),
        you normally won't need to set this unless you are building a special VM
        """
        return pulumi.get(self, "arch")

    @property
    @pulumi.getter
    def autostart(self) -> pulumi.Output[Optional[bool]]:
        """
        Set to `true` to start the domain on host boot up.
        If not specified `false` is assumed.
        """
        return pulumi.get(self, "autostart")

    @property
    @pulumi.getter(name="bootDevices")
    def boot_devices(self) -> pulumi.Output[Optional[Sequence['outputs.DomainBootDevice']]]:
        """
        A list of devices (dev) which defines boot order. Example
        below.
        """
        return pulumi.get(self, "boot_devices")

    @property
    @pulumi.getter
    def cloudinit(self) -> pulumi.Output[Optional[str]]:
        """
        The `CloudInitDisk` disk that has to be used by
        the domain. This is going to be attached as a CDROM ISO. Changing the
        cloud-init won't cause the domain to be recreated, however the change will
        have effect on the next reboot.
        """
        return pulumi.get(self, "cloudinit")

    @property
    @pulumi.getter
    def cmdlines(self) -> pulumi.Output[Optional[Sequence[Mapping[str, Any]]]]:
        """
        Arguments to the kernel
        """
        return pulumi.get(self, "cmdlines")

    @property
    @pulumi.getter
    def consoles(self) -> pulumi.Output[Optional[Sequence['outputs.DomainConsole']]]:
        return pulumi.get(self, "consoles")

    @property
    @pulumi.getter(name="coreosIgnition")
    def coreos_ignition(self) -> pulumi.Output[Optional[str]]:
        """
        The
        [Ignition](https://www.terraform.io/docs/providers/libvirt/r/coreos_ignition.html) resource
        that is to be used by the CoreOS domain.
        """
        return pulumi.get(self, "coreos_ignition")

    @property
    @pulumi.getter
    def cpu(self) -> pulumi.Output[Optional['outputs.DomainCpu']]:
        """
        Configures CPU mode. See below for more
        details.
        """
        return pulumi.get(self, "cpu")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        The description for domain.
        Changing this forces a new resource to be created.
        This data is not used by libvirt in any way, it can contain any information the user wants.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def disks(self) -> pulumi.Output[Optional[Sequence['outputs.DomainDisk']]]:
        """
        An array of one or more disks to attach to the domain. The
        `disk` object structure is documented below.
        """
        return pulumi.get(self, "disks")

    @property
    @pulumi.getter
    def emulator(self) -> pulumi.Output[str]:
        """
        The path of the emulator to use
        """
        return pulumi.get(self, "emulator")

    @property
    @pulumi.getter
    def filesystems(self) -> pulumi.Output[Optional[Sequence['outputs.DomainFilesystem']]]:
        """
        An array of one or more host filesystems to attach to
        the domain. The `filesystem` object structure is documented
        below.
        """
        return pulumi.get(self, "filesystems")

    @property
    @pulumi.getter
    def firmware(self) -> pulumi.Output[Optional[str]]:
        """
        The UEFI rom images for exercising UEFI secure boot in a qemu
        environment. Users should usually specify one of the standard _Open Virtual Machine
        Firmware_ (_OVMF_) images available for their distributions. The file will be opened
        read-only.
        """
        return pulumi.get(self, "firmware")

    @property
    @pulumi.getter(name="fwCfgName")
    def fw_cfg_name(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the firmware config path where ignition file is stored: default is `opt/com.coreos/config`. If you are using [Flatcar Linux](https://docs.flatcar-linux.org/os/booting-with-libvirt/#creating-the-domain-xml), the value is `opt/org.flatcar-linux/config`.
        """
        return pulumi.get(self, "fw_cfg_name")

    @property
    @pulumi.getter
    def graphics(self) -> pulumi.Output[Optional['outputs.DomainGraphics']]:
        return pulumi.get(self, "graphics")

    @property
    @pulumi.getter
    def initrd(self) -> pulumi.Output[Optional[str]]:
        """
        The path of the initrd to boot.
        """
        return pulumi.get(self, "initrd")

    @property
    @pulumi.getter
    def kernel(self) -> pulumi.Output[Optional[str]]:
        """
        The path of the kernel to boot
        """
        return pulumi.get(self, "kernel")

    @property
    @pulumi.getter
    def machine(self) -> pulumi.Output[str]:
        """
        The machine type,
        you normally won't need to set this unless you are running on a platform that
        defaults to the wrong machine type for your template
        """
        return pulumi.get(self, "machine")

    @property
    @pulumi.getter
    def memory(self) -> pulumi.Output[Optional[int]]:
        """
        The amount of memory in MiB. If not specified the domain
        will be created with 512 MiB of memory be used.
        """
        return pulumi.get(self, "memory")

    @property
    @pulumi.getter
    def metadata(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "metadata")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="networkInterfaces")
    def network_interfaces(self) -> pulumi.Output[Optional[Sequence['outputs.DomainNetworkInterface']]]:
        """
        An array of one or more network interfaces to
        attach to the domain. The `network_interface` object structure is documented
        below.
        """
        return pulumi.get(self, "network_interfaces")

    @property
    @pulumi.getter
    def nvram(self) -> pulumi.Output[Optional['outputs.DomainNvram']]:
        """
        this block allows specifying the following attributes related to the _nvram_:
        """
        return pulumi.get(self, "nvram")

    @property
    @pulumi.getter(name="qemuAgent")
    def qemu_agent(self) -> pulumi.Output[Optional[bool]]:
        """
        By default is disabled, set to true for enabling it. More info [qemu-agent](https://wiki.libvirt.org/page/Qemu_guest_agent).
        """
        return pulumi.get(self, "qemu_agent")

    @property
    @pulumi.getter
    def running(self) -> pulumi.Output[Optional[bool]]:
        """
        Use `false` to turn off the instance. If not specified,
        true is assumed and the instance, if stopped, will be started at next apply.
        """
        return pulumi.get(self, "running")

    @property
    @pulumi.getter
    def tpm(self) -> pulumi.Output[Optional['outputs.DomainTpm']]:
        """
        TPM device to attach to the domain. The `tpm` object structure is documented below.
        """
        return pulumi.get(self, "tpm")

    @property
    @pulumi.getter
    def vcpu(self) -> pulumi.Output[Optional[int]]:
        """
        The amount of virtual CPUs. If not specified, a single CPU
        will be created.
        """
        return pulumi.get(self, "vcpu")

    @property
    @pulumi.getter
    def video(self) -> pulumi.Output[Optional['outputs.DomainVideo']]:
        return pulumi.get(self, "video")

    @property
    @pulumi.getter
    def xml(self) -> pulumi.Output[Optional['outputs.DomainXml']]:
        return pulumi.get(self, "xml")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetNetworkDnsHostTemplateResult',
    'AwaitableGetNetworkDnsHostTemplateResult',
    'get_network_dns_host_template',
    'get_network_dns_host_template_output',
]

@pulumi.output_type
class GetNetworkDnsHostTemplateResult:
    """
    A collection of values returned by getNetworkDnsHostTemplate.
    """
    def __init__(__self__, hostname=None, id=None, ip=None, rendered=None):
        if hostname and not isinstance(hostname, str):
            raise TypeError("Expected argument 'hostname' to be a str")
        pulumi.set(__self__, "hostname", hostname)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip and not isinstance(ip, str):
            raise TypeError("Expected argument 'ip' to be a str")
        pulumi.set(__self__, "ip", ip)
        if rendered and not isinstance(rendered, dict):
            raise TypeError("Expected argument 'rendered' to be a dict")
        pulumi.set(__self__, "rendered", rendered)

    @property
    @pulumi.getter
    def hostname(self) -> str:
        return pulumi.get(self, "hostname")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def ip(self) -> str:
        return pulumi.get(self, "ip")

    @property
    @pulumi.getter
    def rendered(self) -> Mapping[str, str]:
        return pulumi.get(self, "rendered")


class AwaitableGetNetworkDnsHostTemplateResult(GetNetworkDnsHostTemplateResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNetworkDnsHostTemplateResult(
            hostname=self.hostname,
            id=self.id,
            ip=self.ip,
            rendered=self.rendered)


def get_network_dns_host_template(hostname: Optional[str] = None,
                                  ip: Optional[str] = None,
                                  opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNetworkDnsHostTemplateResult:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['hostname'] = hostname
    __args__['ip'] = ip
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('libvirt:index/getNetworkDnsHostTemplate:getNetworkDnsHostTemplate', __args__, opts=opts, typ=GetNetworkDnsHostTemplateResult).value

    return AwaitableGetNetworkDnsHostTemplateResult(
        hostname=__ret__.hostname,
        id=__ret__.id,
        ip=__ret__.ip,
        rendered=__ret__.rendered)


@_utilities.lift_output_func(get_network_dns_host_template)
def get_network_dns_host_template_output(hostname: Optional[pulumi.Input[str]] = None,
                                         ip: Optional[pulumi.Input[str]] = None,
                                         opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetNetworkDnsHostTemplateResult]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['VolumeArgs', 'Volume']

@pulumi.input_type
class VolumeArgs:
    def __init__(__self__, *,
                 base_volume_id: Optional[pulumi.Input[str]] = None,
                 base_volume_name: Optional[pulumi.Input[str]] = None,
                 base_volume_pool: Optional[pulumi.Input[str]] = None,
                 format: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None,
                 size: Optional[pulumi.Input[int]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 xml: Optional[pulumi.Input['VolumeXmlArgs']] = None):
        """
        The set of arguments for constructing a Volume resource.
        :param pulumi.Input[str] base_volume_id: The backing volume (CoW) to use for this volume.
        :param pulumi.Input[str] base_volume_name: The name of the backing volume (CoW) to use
               for this volume. Note well: when `base_volume_pool` is not specified the
               volume is going to be searched inside of `pool`.
        :param pulumi.Input[str] base_volume_pool: The name of the storage pool containing the
               volume defined by `base_volume_name`.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[str] pool: The storage pool where the resource will be created.
               If not given, the `default` storage pool will be used.
        """
        if base_volume_id is not None:
            pulumi.set(__self__, "base_volume_id", base_volume_id)
        if base_volume_name is not None:
            pulumi.set(__self__, "base_volume_name", base_volume_name)
        if base_volume_pool is not None:
            pulumi.set(__self__, "base_volume_pool", base_volume_pool)
        if format is not None:
            pulumi.set(__self__, "format", format)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)
        if size is not None:
            pulumi.set(__self__, "size", size)
        if source is not None:
            pulumi.set(__self__, "source", source)
        if xml is not None:
            pulumi.set(__self__, "xml", xml)

    @property
    @pulumi.getter(name="baseVolumeId")
    def base_volume_id(self) -> Optional[pulumi.Input[str]]:
        """
        The backing volume (CoW) to use for this volume.
        """
        return pulumi.get(self, "base_volume_id")

    @base_volume_id.setter
    def base_volume_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "base_volume_id", value)

    @property
    @pulumi.getter(name="baseVolumeName")
    def base_volume_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the backing volume (CoW) to use
        for this volume. Note well: when `base_volume_pool` is not specified the
        volume is going to be searched inside of `pool`.
        """
        return pulumi.get(self, "base_volume_name")

    @base_volume_name.setter
    def base_volume_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "base_volume_name", value)

    @property
    @pulumi.getter(name="baseVolumePool")
    def base_volume_pool(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the storage pool containing the
        volume defined by `base_volume_name`.
        """
        return pulumi.get(self, "base_volume_pool")

    @base_volume_pool.setter
    def base_volume_pool(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "base_volume_pool", value)

    @property
    @pulumi.getter
    def format(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "format")

    @format.setter
    def format(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "format", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def pool(self) -> Optional[pulumi.Input[str]]:
        """
        The storage pool where the resource will be created.
        If not given, the `default` storage pool will be used.
        """
        return pulumi.get(self, "pool")

    @pool.setter
    def pool(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pool", value)

    @property
    @pulumi.getter
    def size(self) -> Optional[pulumi.Input[int]]:
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "size", value)

    @property
    @pulumi.getter
    def source(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source", value)

    @property
    @pulumi.getter
    def xml(self) -> Optional[pulumi.Input['VolumeXmlArgs']]:
        return pulumi.get(self, "xml")

    @xml.setter
    def xml(self, value: Optional[pulumi.Input['VolumeXmlArgs']]):
        pulumi.set(self, "xml", value)


@pulumi.input_type
class _VolumeState:
    def __init__(__self__, *,
                 base_volume_id: Optional[pulumi.Input[str]] = None,
                 base_volume_name: Optional[pulumi.Input[str]] = None,
                 base_volume_pool: Optional[pulumi.Input[str]] = None,
                 format: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None,
                 size: Optional[pulumi.Input[int]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 xml: Optional[pulumi.Input['VolumeXmlArgs']] = None):
        """
        Input properties used for looking up and filtering Volume resources.
        :param pulumi.Input[str] base_volume_id: The backing volume (CoW) to use for this volume.
        :param pulumi.Input[str] base_volume_name: The name of the backing volume (CoW) to use
               for this volume. Note well: when `base_volume_pool` is not specified the
               volume is going to be searched inside of `pool`.
        :param pulumi.Input[str] base_volume_pool: The name of the storage pool containing the
               volume defined by `base_volume_name`.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[str] pool: The storage pool where the resource will be created.
               If not given, the `default` storage pool will be used.
        """
        if base_volume_id is not None:
            pulumi.set(__self__, "base_volume_id", base_volume_id)
        if base_volume_name is not None:
            pulumi.set(__self__, "base_volume_name", base_volume_name)
        if base_volume_pool is not None:
            pulumi.set(__self__, "base_volume_pool", base_volume_pool)
        if format is not None:
            pulumi.set(__self__, "format", format)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)
        if size is not None:
            pulumi.set(__self__, "size", size)
        if source is not None:
            pulumi.set(__self__, "source", source)
        if xml is not None:
            pulumi.set(__self__, "xml", xml)

    @property
    @pulumi.getter(name="baseVolumeId")
    def base_volume_id(self) -> Optional[pulumi.Input[str]]:
        """
        The backing volume (CoW) to use for this volume.
        """
        return pulumi.get(self, "base_volume_id")

    @base_volume_id.setter
    def base_volume_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "base_volume_id", value)

    @property
    @pulumi.getter(name="baseVolumeName")
    def base_volume_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the backing volume (CoW) to use
        for this volume. Note well: when `base_volume_pool` is not specified the
        volume is going to be searched inside of `pool`.
        """
        return pulumi.get(self, "base_volume_name")

    @base_volume_name.setter
    def base_volume_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "base_volume_name", value)

    @property
    @pulumi.getter(name="baseVolumePool")
    def base_volume_pool(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the storage pool containing the
        volume defined by `base_volume_name`.
        """
        return pulumi.get(self, "base_volume_pool")

    @base_volume_pool.setter
    def base_volume_pool(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "base_volume_pool", value)

    @property
    @pulumi.getter
    def format(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "format")

    @format.setter
    def format(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "format", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def pool(self) -> Optional[pulumi.Input[str]]:
        """
        The storage pool where the resource will be created.
        If not given, the `default` storage pool will be used.
        """
        return pulumi.get(self, "pool")

    @pool.setter
    def pool(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pool", value)

    @property
    @pulumi.getter
    def size(self) -> Optional[pulumi.Input[int]]:
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "size", value)

    @property
    @pulumi.getter
    def source(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source", value)

    @property
    @pulumi.getter
    def xml(self) -> Optional[pulumi.Input['VolumeXmlArgs']]:
        return pulumi.get(self, "xml")

    @xml.setter
    def xml(self, value: Optional[pulumi.Input['VolumeXmlArgs']]):
        pulumi.set(self, "xml", value)


class Volume(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 base_volume_id: Optional[pulumi.Input[str]] = None,
                 base_volume_name: Optional[pulumi.Input[str]] = None,
                 base_volume_pool: Optional[pulumi.Input[str]] = None,
                 format: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None,
                 size: Optional[pulumi.Input[int]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 xml: Optional[pulumi.Input[pulumi.InputType['VolumeXmlArgs']]] = None,
                 __props__=None):
        """
        Manages a storage volume in libvirt. For more information see
        [the official documentation](https://libvirt.org/formatstorage.html).

        ## Example Usage

        ```python
        import pulumi
        import pulumi_libvirt as libvirt

        # Base OS image to use to create a cluster of different
        # nodes
        opensuse_leap = libvirt.Volume("opensuseLeap", source="http://download.opensuse.org/repositories/Cloud:/Images:/Leap_42.1/images/openSUSE-Leap-42.1-OpenStack.x86_64.qcow2")
        # volume to attach to the "master" domain as main disk
        master = libvirt.Volume("master", base_volume_id=opensuse_leap.id)
        # volumes to attach to the "workers" domains as main disk
        worker = []
        for range in [{"value": i} for i in range(0, var.workers_count)]:
            worker.append(libvirt.Volume(f"worker-{range['value']}", base_volume_id=opensuse_leap.id))
        ```

        > **Tip:** when provisioning multiple domains using the same base image, create
        a `Volume` for the base image and then define the domain specific ones
        as based on it. This way the image will not be modified and no extra disk space
        is going to be used for the base image.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] base_volume_id: The backing volume (CoW) to use for this volume.
        :param pulumi.Input[str] base_volume_name: The name of the backing volume (CoW) to use
               for this volume. Note well: when `base_volume_pool` is not specified the
               volume is going to be searched inside of `pool`.
        :param pulumi.Input[str] base_volume_pool: The name of the storage pool containing the
               volume defined by `base_volume_name`.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[str] pool: The storage pool where the resource will be created.
               If not given, the `default` storage pool will be used.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[VolumeArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a storage volume in libvirt. For more information see
        [the official documentation](https://libvirt.org/formatstorage.html).

        ## Example Usage

        ```python
        import pulumi
        import pulumi_libvirt as libvirt

        # Base OS image to use to create a cluster of different
        # nodes
        opensuse_leap = libvirt.Volume("opensuseLeap", source="http://download.opensuse.org/repositories/Cloud:/Images:/Leap_42.1/images/openSUSE-Leap-42.1-OpenStack.x86_64.qcow2")
        # volume to attach to the "master" domain as main disk
        master = libvirt.Volume("master", base_volume_id=opensuse_leap.id)
        # volumes to attach to the "workers" domains as main disk
        worker = []
        for range in [{"value": i} for i in range(0, var.workers_count)]:
            worker.append(libvirt.Volume(f"worker-{range['value']}", base_volume_id=opensuse_leap.id))
        ```

        > **Tip:** when provisioning multiple domains using the same base image, create
        a `Volume` for the base image and then define the domain specific ones
        as based on it. This way the image will not be modified and no extra disk space
        is going to be used for the base image.

        :param str resource_name: The name of the resource.
        :param VolumeArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(VolumeArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 base_volume_id: Optional[pulumi.Input[str]] = None,
                 base_volume_name: Optional[pulumi.Input[str]] = None,
                 base_volume_pool: Optional[pulumi.Input[str]] = None,
                 format: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None,
                 size: Optional[pulumi.Input[int]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 xml: Optional[pulumi.Input[pulumi.InputType['VolumeXmlArgs']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = VolumeArgs.__new__(VolumeArgs)

            __props__.__dict__["base_volume_id"] = base_volume_id
            __props__.__dict__["base_volume_name"] = base_volume_name
            __props__.__dict__["base_volume_pool"] = base_volume_pool
            __props__.__dict__["format"] = format
            __props__.__dict__["name"] = name
            __props__.__dict__["pool"] = pool
            __props__.__dict__["size"] = size
            __props__.__dict__["source"] = source
            __props__.__dict__["xml"] = xml
        super(Volume, __self__).__init__(
            'libvirt:index/volume:Volume',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            base_volume_id: Optional[pulumi.Input[str]] = None,
            base_volume_name: Optional[pulumi.Input[str]] = None,
            base_volume_pool: Optional[pulumi.Input[str]] = None,
            format: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            pool: Optional[pulumi.Input[str]] = None,
            size: Optional[pulumi.Input[int]] = None,
            source: Optional[pulumi.Input[str]] = None,
            xml: Optional[pulumi.Input[pulumi.InputType['VolumeXmlArgs']]] = None) -> 'Volume':
        """
        Get an existing Volume resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] base_volume_id: The backing volume (CoW) to use for this volume.
        :param pulumi.Input[str] base_volume_name: The name of the backing volume (CoW) to use
               for this volume. Note well: when `base_volume_pool` is not specified the
               volume is going to be searched inside of `pool`.
        :param pulumi.Input[str] base_volume_pool: The name of the storage pool containing the
               volume defined by `base_volume_name`.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[str] pool: The storage pool where the resource will be created.
               If not given, the `default` storage pool will be used.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _VolumeState.__new__(_VolumeState)

        __props__.__dict__["base_volume_id"] = base_volume_id
        __props__.__dict__["base_volume_name"] = base_volume_name
        __props__.__dict__["base_volume_pool"] = base_volume_pool
        __props__.__dict__["format"] = format
        __props__.__dict__["name"] = name
        __props__.__dict__["pool"] = pool
        __props__.__dict__["size"] = size
        __props__.__dict__["source"] = source
        __props__.__dict__["xml"] = xml
        return Volume(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="baseVolumeId")
    def base_volume_id(self) -> pulumi.Output[Optional[str]]:
        """
        The backing volume (CoW) to use for this volume.
        """
        return pulumi.get(self, "base_volume_id")

    @property
    @pulumi.getter(name="baseVolumeName")
    def base_volume_name(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the backing volume (CoW) to use
        for this volume. Note well: when `base_volume_pool` is not specified the
        volume is going to be searched inside of `pool`.
        """
        return pulumi.get(self, "base_volume_name")

    @property
    @pulumi.getter(name="baseVolumePool")
    def base_volume_pool(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the storage pool containing the
        volume defined by `base_volume_name`.
        """
        return pulumi.get(self, "base_volume_pool")

    @property
    @pulumi.getter
    def format(self) -> pulumi.Output[str]:
        return pulumi.get(self, "format")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def pool(self) -> pulumi.Output[Optional[str]]:
        """
        The storage pool where the resource will be created.
        If not given, the `default` storage pool will be used.
        """
        return pulumi.get(self, "pool")

    @property
    @pulumi.getter
    def size(self) -> pulumi.Output[int]:
        return pulumi.get(self, "size")

    @property
    @pulumi.getter
    def source(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "source")

    @property
    @pulumi.getter
    def xml(self) -> pulumi.Output[Optional['outputs.VolumeXml']]:
        return pulumi.get(self, "xml")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetNetworkDnsmasqOptionsTemplateResult',
    'AwaitableGetNetworkDnsmasqOptionsTemplateResult',
    'get_network_dnsmasq_options_template',
    'get_network_dnsmasq_options_template_output',
]

@pulumi.output_type
class GetNetworkDnsmasqOptionsTemplateResult:
    """
    A collection of values returned by getNetworkDnsmasqOptionsTemplate.
    """
    def __init__(__self__, id=None, option_name=None, option_value=None, rendered=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if option_name and not isinstance(option_name, str):
            raise TypeError("Expected argument 'option_name' to be a str")
        pulumi.set(__self__, "option_name", option_name)
        if option_value and not isinstance(option_value, str):
            raise TypeError("Expected argument 'option_value' to be a str")
        pulumi.set(__self__, "option_value", option_value)
        if rendered and not isinstance(rendered, dict):
            raise TypeError("Expected argument 'rendered' to be a dict")
        pulumi.set(__self__, "rendered", rendered)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="optionName")
    def option_name(self) -> str:
        return pulumi.get(self, "option_name")

    @property
    @pulumi.getter(name="optionValue")
    def option_value(self) -> str:
        return pulumi.get(self, "option_value")

    @property
    @pulumi.getter
    def rendered(self) -> Mapping[str, str]:
        return pulumi.get(self, "rendered")


class AwaitableGetNetworkDnsmasqOptionsTemplateResult(GetNetworkDnsmasqOptionsTemplateResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNetworkDnsmasqOptionsTemplateResult(
            id=self.id,
            option_name=self.option_name,
            option_value=self.option_value,
            rendered=self.rendered)


def get_network_dnsmasq_options_template(option_name: Optional[str] = None,
                                         option_value: Optional[str] = None,
                                         opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNetworkDnsmasqOptionsTemplateResult:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['optionName'] = option_name
    __args__['optionValue'] = option_value
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('libvirt:index/getNetworkDnsmasqOptionsTemplate:getNetworkDnsmasqOptionsTemplate', __args__, opts=opts, typ=GetNetworkDnsmasqOptionsTemplateResult).value

    return AwaitableGetNetworkDnsmasqOptionsTemplateResult(
        id=__ret__.id,
        option_name=__ret__.option_name,
        option_value=__ret__.option_value,
        rendered=__ret__.rendered)


@_utilities.lift_output_func(get_network_dnsmasq_options_template)
def get_network_dnsmasq_options_template_output(option_name: Optional[pulumi.Input[str]] = None,
                                                option_value: Optional[pulumi.Input[str]] = None,
                                                opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetNetworkDnsmasqOptionsTemplateResult]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['NetworkArgs', 'Network']

@pulumi.input_type
class NetworkArgs:
    def __init__(__self__, *,
                 addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 bridge: Optional[pulumi.Input[str]] = None,
                 dhcp: Optional[pulumi.Input['NetworkDhcpArgs']] = None,
                 dns: Optional[pulumi.Input['NetworkDnsArgs']] = None,
                 dnsmasq_options: Optional[pulumi.Input['NetworkDnsmasqOptionsArgs']] = None,
                 domain: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mtu: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 routes: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]]] = None,
                 xml: Optional[pulumi.Input['NetworkXmlArgs']] = None):
        """
        The set of arguments for constructing a Network resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] addresses: A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
               CIDR notation.  This defines the subnets associated to that network.
               This argument is also used to define the address on the real host.
               If `dhcp {  enabled = true }` addresses is also used to define the address range served by
               the DHCP server.
               No DHCP server will be started if `addresses` is omitted.
        :param pulumi.Input[bool] autostart: Set to `true` to start the network on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[str] bridge: The bridge device defines the name of a bridge
               device which will be used to construct the virtual network (when not provided,
               it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        :param pulumi.Input['NetworkDhcpArgs'] dhcp: DHCP configuration. 
               You need to use it in conjuction with the adresses variable.
        :param pulumi.Input['NetworkDnsArgs'] dns: configuration of DNS specific settings for the network
        :param pulumi.Input['NetworkDnsmasqOptionsArgs'] dnsmasq_options: configuration of Dnsmasq options for the network
               You need to provide a list of option name and value pairs.
        :param pulumi.Input[str] domain: The domain used by the DNS server.
        :param pulumi.Input[str] mode: One of:
               - `none`: the guests can talk to each other and the host OS, but cannot reach
               any other machines on the LAN.
               - `nat`: it is the default network mode. This is a configuration that
               allows guest OS to get outbound connectivity regardless of whether the host
               uses ethernet, wireless, dialup, or VPN networking without requiring any
               specific admin configuration. In the absence of host networking, it at
               least allows guests to talk directly to each other.
               - `route`: this is a variant on the default network which routes traffic from
               the virtual network to the LAN **without applying any NAT**. It requires that
               the IP address range be pre-configured in the routing tables of the router
               on the host network.
               - `open`: similar to `route`, but no firewall rules are added.
               - `bridge`: use a pre-existing host bridge. The guests will effectively be
               directly connected to the physical network (i.e. their IP addresses will
               all be on the subnet of the physical network, and there will be no
               restrictions on inbound or outbound connections). The `bridge` network
               attribute is mandatory in this case.
        :param pulumi.Input[int] mtu: The MTU to set for the underlying network interfaces. When
               not supplied, libvirt will use the default for the interface, usually 1500.
               Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]] routes: a list of static routes. A `cidr` and a `gateway` must
               be provided. The `gateway` must be reachable via the bridge interface.
        """
        if addresses is not None:
            pulumi.set(__self__, "addresses", addresses)
        if autostart is not None:
            pulumi.set(__self__, "autostart", autostart)
        if bridge is not None:
            pulumi.set(__self__, "bridge", bridge)
        if dhcp is not None:
            pulumi.set(__self__, "dhcp", dhcp)
        if dns is not None:
            pulumi.set(__self__, "dns", dns)
        if dnsmasq_options is not None:
            pulumi.set(__self__, "dnsmasq_options", dnsmasq_options)
        if domain is not None:
            pulumi.set(__self__, "domain", domain)
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if mtu is not None:
            pulumi.set(__self__, "mtu", mtu)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if routes is not None:
            pulumi.set(__self__, "routes", routes)
        if xml is not None:
            pulumi.set(__self__, "xml", xml)

    @property
    @pulumi.getter
    def addresses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
        CIDR notation.  This defines the subnets associated to that network.
        This argument is also used to define the address on the real host.
        If `dhcp {  enabled = true }` addresses is also used to define the address range served by
        the DHCP server.
        No DHCP server will be started if `addresses` is omitted.
        """
        return pulumi.get(self, "addresses")

    @addresses.setter
    def addresses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "addresses", value)

    @property
    @pulumi.getter
    def autostart(self) -> Optional[pulumi.Input[bool]]:
        """
        Set to `true` to start the network on host boot up.
        If not specified `false` is assumed.
        """
        return pulumi.get(self, "autostart")

    @autostart.setter
    def autostart(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "autostart", value)

    @property
    @pulumi.getter
    def bridge(self) -> Optional[pulumi.Input[str]]:
        """
        The bridge device defines the name of a bridge
        device which will be used to construct the virtual network (when not provided,
        it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        """
        return pulumi.get(self, "bridge")

    @bridge.setter
    def bridge(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "bridge", value)

    @property
    @pulumi.getter
    def dhcp(self) -> Optional[pulumi.Input['NetworkDhcpArgs']]:
        """
        DHCP configuration. 
        You need to use it in conjuction with the adresses variable.
        """
        return pulumi.get(self, "dhcp")

    @dhcp.setter
    def dhcp(self, value: Optional[pulumi.Input['NetworkDhcpArgs']]):
        pulumi.set(self, "dhcp", value)

    @property
    @pulumi.getter
    def dns(self) -> Optional[pulumi.Input['NetworkDnsArgs']]:
        """
        configuration of DNS specific settings for the network
        """
        return pulumi.get(self, "dns")

    @dns.setter
    def dns(self, value: Optional[pulumi.Input['NetworkDnsArgs']]):
        pulumi.set(self, "dns", value)

    @property
    @pulumi.getter(name="dnsmasqOptions")
    def dnsmasq_options(self) -> Optional[pulumi.Input['NetworkDnsmasqOptionsArgs']]:
        """
        configuration of Dnsmasq options for the network
        You need to provide a list of option name and value pairs.
        """
        return pulumi.get(self, "dnsmasq_options")

    @dnsmasq_options.setter
    def dnsmasq_options(self, value: Optional[pulumi.Input['NetworkDnsmasqOptionsArgs']]):
        pulumi.set(self, "dnsmasq_options", value)

    @property
    @pulumi.getter
    def domain(self) -> Optional[pulumi.Input[str]]:
        """
        The domain used by the DNS server.
        """
        return pulumi.get(self, "domain")

    @domain.setter
    def domain(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "domain", value)

    @property
    @pulumi.getter
    def mode(self) -> Optional[pulumi.Input[str]]:
        """
        One of:
        - `none`: the guests can talk to each other and the host OS, but cannot reach
        any other machines on the LAN.
        - `nat`: it is the default network mode. This is a configuration that
        allows guest OS to get outbound connectivity regardless of whether the host
        uses ethernet, wireless, dialup, or VPN networking without requiring any
        specific admin configuration. In the absence of host networking, it at
        least allows guests to talk directly to each other.
        - `route`: this is a variant on the default network which routes traffic from
        the virtual network to the LAN **without applying any NAT**. It requires that
        the IP address range be pre-configured in the routing tables of the router
        on the host network.
        - `open`: similar to `route`, but no firewall rules are added.
        - `bridge`: use a pre-existing host bridge. The guests will effectively be
        directly connected to the physical network (i.e. their IP addresses will
        all be on the subnet of the physical network, and there will be no
        restrictions on inbound or outbound connections). The `bridge` network
        attribute is mandatory in this case.
        """
        return pulumi.get(self, "mode")

    @mode.setter
    def mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mode", value)

    @property
    @pulumi.getter
    def mtu(self) -> Optional[pulumi.Input[int]]:
        """
        The MTU to set for the underlying network interfaces. When
        not supplied, libvirt will use the default for the interface, usually 1500.
        Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        """
        return pulumi.get(self, "mtu")

    @mtu.setter
    def mtu(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "mtu", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def routes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]]]:
        """
        a list of static routes. A `cidr` and a `gateway` must
        be provided. The `gateway` must be reachable via the bridge interface.
        """
        return pulumi.get(self, "routes")

    @routes.setter
    def routes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]]]):
        pulumi.set(self, "routes", value)

    @property
    @pulumi.getter
    def xml(self) -> Optional[pulumi.Input['NetworkXmlArgs']]:
        return pulumi.get(self, "xml")

    @xml.setter
    def xml(self, value: Optional[pulumi.Input['NetworkXmlArgs']]):
        pulumi.set(self, "xml", value)


@pulumi.input_type
class _NetworkState:
    def __init__(__self__, *,
                 addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 bridge: Optional[pulumi.Input[str]] = None,
                 dhcp: Optional[pulumi.Input['NetworkDhcpArgs']] = None,
                 dns: Optional[pulumi.Input['NetworkDnsArgs']] = None,
                 dnsmasq_options: Optional[pulumi.Input['NetworkDnsmasqOptionsArgs']] = None,
                 domain: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mtu: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 routes: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]]] = None,
                 xml: Optional[pulumi.Input['NetworkXmlArgs']] = None):
        """
        Input properties used for looking up and filtering Network resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] addresses: A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
               CIDR notation.  This defines the subnets associated to that network.
               This argument is also used to define the address on the real host.
               If `dhcp {  enabled = true }` addresses is also used to define the address range served by
               the DHCP server.
               No DHCP server will be started if `addresses` is omitted.
        :param pulumi.Input[bool] autostart: Set to `true` to start the network on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[str] bridge: The bridge device defines the name of a bridge
               device which will be used to construct the virtual network (when not provided,
               it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        :param pulumi.Input['NetworkDhcpArgs'] dhcp: DHCP configuration. 
               You need to use it in conjuction with the adresses variable.
        :param pulumi.Input['NetworkDnsArgs'] dns: configuration of DNS specific settings for the network
        :param pulumi.Input['NetworkDnsmasqOptionsArgs'] dnsmasq_options: configuration of Dnsmasq options for the network
               You need to provide a list of option name and value pairs.
        :param pulumi.Input[str] domain: The domain used by the DNS server.
        :param pulumi.Input[str] mode: One of:
               - `none`: the guests can talk to each other and the host OS, but cannot reach
               any other machines on the LAN.
               - `nat`: it is the default network mode. This is a configuration that
               allows guest OS to get outbound connectivity regardless of whether the host
               uses ethernet, wireless, dialup, or VPN networking without requiring any
               specific admin configuration. In the absence of host networking, it at
               least allows guests to talk directly to each other.
               - `route`: this is a variant on the default network which routes traffic from
               the virtual network to the LAN **without applying any NAT**. It requires that
               the IP address range be pre-configured in the routing tables of the router
               on the host network.
               - `open`: similar to `route`, but no firewall rules are added.
               - `bridge`: use a pre-existing host bridge. The guests will effectively be
               directly connected to the physical network (i.e. their IP addresses will
               all be on the subnet of the physical network, and there will be no
               restrictions on inbound or outbound connections). The `bridge` network
               attribute is mandatory in this case.
        :param pulumi.Input[int] mtu: The MTU to set for the underlying network interfaces. When
               not supplied, libvirt will use the default for the interface, usually 1500.
               Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]] routes: a list of static routes. A `cidr` and a `gateway` must
               be provided. The `gateway` must be reachable via the bridge interface.
        """
        if addresses is not None:
            pulumi.set(__self__, "addresses", addresses)
        if autostart is not None:
            pulumi.set(__self__, "autostart", autostart)
        if bridge is not None:
            pulumi.set(__self__, "bridge", bridge)
        if dhcp is not None:
            pulumi.set(__self__, "dhcp", dhcp)
        if dns is not None:
            pulumi.set(__self__, "dns", dns)
        if dnsmasq_options is not None:
            pulumi.set(__self__, "dnsmasq_options", dnsmasq_options)
        if domain is not None:
            pulumi.set(__self__, "domain", domain)
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if mtu is not None:
            pulumi.set(__self__, "mtu", mtu)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if routes is not None:
            pulumi.set(__self__, "routes", routes)
        if xml is not None:
            pulumi.set(__self__, "xml", xml)

    @property
    @pulumi.getter
    def addresses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
        CIDR notation.  This defines the subnets associated to that network.
        This argument is also used to define the address on the real host.
        If `dhcp {  enabled = true }` addresses is also used to define the address range served by
        the DHCP server.
        No DHCP server will be started if `addresses` is omitted.
        """
        return pulumi.get(self, "addresses")

    @addresses.setter
    def addresses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "addresses", value)

    @property
    @pulumi.getter
    def autostart(self) -> Optional[pulumi.Input[bool]]:
        """
        Set to `true` to start the network on host boot up.
        If not specified `false` is assumed.
        """
        return pulumi.get(self, "autostart")

    @autostart.setter
    def autostart(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "autostart", value)

    @property
    @pulumi.getter
    def bridge(self) -> Optional[pulumi.Input[str]]:
        """
        The bridge device defines the name of a bridge
        device which will be used to construct the virtual network (when not provided,
        it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        """
        return pulumi.get(self, "bridge")

    @bridge.setter
    def bridge(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "bridge", value)

    @property
    @pulumi.getter
    def dhcp(self) -> Optional[pulumi.Input['NetworkDhcpArgs']]:
        """
        DHCP configuration. 
        You need to use it in conjuction with the adresses variable.
        """
        return pulumi.get(self, "dhcp")

    @dhcp.setter
    def dhcp(self, value: Optional[pulumi.Input['NetworkDhcpArgs']]):
        pulumi.set(self, "dhcp", value)

    @property
    @pulumi.getter
    def dns(self) -> Optional[pulumi.Input['NetworkDnsArgs']]:
        """
        configuration of DNS specific settings for the network
        """
        return pulumi.get(self, "dns")

    @dns.setter
    def dns(self, value: Optional[pulumi.Input['NetworkDnsArgs']]):
        pulumi.set(self, "dns", value)

    @property
    @pulumi.getter(name="dnsmasqOptions")
    def dnsmasq_options(self) -> Optional[pulumi.Input['NetworkDnsmasqOptionsArgs']]:
        """
        configuration of Dnsmasq options for the network
        You need to provide a list of option name and value pairs.
        """
        return pulumi.get(self, "dnsmasq_options")

    @dnsmasq_options.setter
    def dnsmasq_options(self, value: Optional[pulumi.Input['NetworkDnsmasqOptionsArgs']]):
        pulumi.set(self, "dnsmasq_options", value)

    @property
    @pulumi.getter
    def domain(self) -> Optional[pulumi.Input[str]]:
        """
        The domain used by the DNS server.
        """
        return pulumi.get(self, "domain")

    @domain.setter
    def domain(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "domain", value)

    @property
    @pulumi.getter
    def mode(self) -> Optional[pulumi.Input[str]]:
        """
        One of:
        - `none`: the guests can talk to each other and the host OS, but cannot reach
        any other machines on the LAN.
        - `nat`: it is the default network mode. This is a configuration that
        allows guest OS to get outbound connectivity regardless of whether the host
        uses ethernet, wireless, dialup, or VPN networking without requiring any
        specific admin configuration. In the absence of host networking, it at
        least allows guests to talk directly to each other.
        - `route`: this is a variant on the default network which routes traffic from
        the virtual network to the LAN **without applying any NAT**. It requires that
        the IP address range be pre-configured in the routing tables of the router
        on the host network.
        - `open`: similar to `route`, but no firewall rules are added.
        - `bridge`: use a pre-existing host bridge. The guests will effectively be
        directly connected to the physical network (i.e. their IP addresses will
        all be on the subnet of the physical network, and there will be no
        restrictions on inbound or outbound connections). The `bridge` network
        attribute is mandatory in this case.
        """
        return pulumi.get(self, "mode")

    @mode.setter
    def mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mode", value)

    @property
    @pulumi.getter
    def mtu(self) -> Optional[pulumi.Input[int]]:
        """
        The MTU to set for the underlying network interfaces. When
        not supplied, libvirt will use the default for the interface, usually 1500.
        Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        """
        return pulumi.get(self, "mtu")

    @mtu.setter
    def mtu(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "mtu", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def routes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]]]:
        """
        a list of static routes. A `cidr` and a `gateway` must
        be provided. The `gateway` must be reachable via the bridge interface.
        """
        return pulumi.get(self, "routes")

    @routes.setter
    def routes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['NetworkRouteArgs']]]]):
        pulumi.set(self, "routes", value)

    @property
    @pulumi.getter
    def xml(self) -> Optional[pulumi.Input['NetworkXmlArgs']]:
        return pulumi.get(self, "xml")

    @xml.setter
    def xml(self, value: Optional[pulumi.Input['NetworkXmlArgs']]):
        pulumi.set(self, "xml", value)


class Network(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 bridge: Optional[pulumi.Input[str]] = None,
                 dhcp: Optional[pulumi.Input[pulumi.InputType['NetworkDhcpArgs']]] = None,
                 dns: Optional[pulumi.Input[pulumi.InputType['NetworkDnsArgs']]] = None,
                 dnsmasq_options: Optional[pulumi.Input[pulumi.InputType['NetworkDnsmasqOptionsArgs']]] = None,
                 domain: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mtu: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 routes: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['NetworkRouteArgs']]]]] = None,
                 xml: Optional[pulumi.Input[pulumi.InputType['NetworkXmlArgs']]] = None,
                 __props__=None):
        """
        Manages a VM network resource within libvirt. For more information see
        [the official documentation](https://libvirt.org/formatnetwork.html).

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] addresses: A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
               CIDR notation.  This defines the subnets associated to that network.
               This argument is also used to define the address on the real host.
               If `dhcp {  enabled = true }` addresses is also used to define the address range served by
               the DHCP server.
               No DHCP server will be started if `addresses` is omitted.
        :param pulumi.Input[bool] autostart: Set to `true` to start the network on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[str] bridge: The bridge device defines the name of a bridge
               device which will be used to construct the virtual network (when not provided,
               it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        :param pulumi.Input[pulumi.InputType['NetworkDhcpArgs']] dhcp: DHCP configuration. 
               You need to use it in conjuction with the adresses variable.
        :param pulumi.Input[pulumi.InputType['NetworkDnsArgs']] dns: configuration of DNS specific settings for the network
        :param pulumi.Input[pulumi.InputType['NetworkDnsmasqOptionsArgs']] dnsmasq_options: configuration of Dnsmasq options for the network
               You need to provide a list of option name and value pairs.
        :param pulumi.Input[str] domain: The domain used by the DNS server.
        :param pulumi.Input[str] mode: One of:
               - `none`: the guests can talk to each other and the host OS, but cannot reach
               any other machines on the LAN.
               - `nat`: it is the default network mode. This is a configuration that
               allows guest OS to get outbound connectivity regardless of whether the host
               uses ethernet, wireless, dialup, or VPN networking without requiring any
               specific admin configuration. In the absence of host networking, it at
               least allows guests to talk directly to each other.
               - `route`: this is a variant on the default network which routes traffic from
               the virtual network to the LAN **without applying any NAT**. It requires that
               the IP address range be pre-configured in the routing tables of the router
               on the host network.
               - `open`: similar to `route`, but no firewall rules are added.
               - `bridge`: use a pre-existing host bridge. The guests will effectively be
               directly connected to the physical network (i.e. their IP addresses will
               all be on the subnet of the physical network, and there will be no
               restrictions on inbound or outbound connections). The `bridge` network
               attribute is mandatory in this case.
        :param pulumi.Input[int] mtu: The MTU to set for the underlying network interfaces. When
               not supplied, libvirt will use the default for the interface, usually 1500.
               Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['NetworkRouteArgs']]]] routes: a list of static routes. A `cidr` and a `gateway` must
               be provided. The `gateway` must be reachable via the bridge interface.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[NetworkArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a VM network resource within libvirt. For more information see
        [the official documentation](https://libvirt.org/formatnetwork.html).

        :param str resource_name: The name of the resource.
        :param NetworkArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(NetworkArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 autostart: Optional[pulumi.Input[bool]] = None,
                 bridge: Optional[pulumi.Input[str]] = None,
                 dhcp: Optional[pulumi.Input[pulumi.InputType['NetworkDhcpArgs']]] = None,
                 dns: Optional[pulumi.Input[pulumi.InputType['NetworkDnsArgs']]] = None,
                 dnsmasq_options: Optional[pulumi.Input[pulumi.InputType['NetworkDnsmasqOptionsArgs']]] = None,
                 domain: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mtu: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 routes: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['NetworkRouteArgs']]]]] = None,
                 xml: Optional[pulumi.Input[pulumi.InputType['NetworkXmlArgs']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = NetworkArgs.__new__(NetworkArgs)

            __props__.__dict__["addresses"] = addresses
            __props__.__dict__["autostart"] = autostart
            __props__.__dict__["bridge"] = bridge
            __props__.__dict__["dhcp"] = dhcp
            __props__.__dict__["dns"] = dns
            __props__.__dict__["dnsmasq_options"] = dnsmasq_options
            __props__.__dict__["domain"] = domain
            __props__.__dict__["mode"] = mode
            __props__.__dict__["mtu"] = mtu
            __props__.__dict__["name"] = name
            __props__.__dict__["routes"] = routes
            __props__.__dict__["xml"] = xml
        super(Network, __self__).__init__(
            'libvirt:index/network:Network',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            autostart: Optional[pulumi.Input[bool]] = None,
            bridge: Optional[pulumi.Input[str]] = None,
            dhcp: Optional[pulumi.Input[pulumi.InputType['NetworkDhcpArgs']]] = None,
            dns: Optional[pulumi.Input[pulumi.InputType['NetworkDnsArgs']]] = None,
            dnsmasq_options: Optional[pulumi.Input[pulumi.InputType['NetworkDnsmasqOptionsArgs']]] = None,
            domain: Optional[pulumi.Input[str]] = None,
            mode: Optional[pulumi.Input[str]] = None,
            mtu: Optional[pulumi.Input[int]] = None,
            name: Optional[pulumi.Input[str]] = None,
            routes: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['NetworkRouteArgs']]]]] = None,
            xml: Optional[pulumi.Input[pulumi.InputType['NetworkXmlArgs']]] = None) -> 'Network':
        """
        Get an existing Network resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] addresses: A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
               CIDR notation.  This defines the subnets associated to that network.
               This argument is also used to define the address on the real host.
               If `dhcp {  enabled = true }` addresses is also used to define the address range served by
               the DHCP server.
               No DHCP server will be started if `addresses` is omitted.
        :param pulumi.Input[bool] autostart: Set to `true` to start the network on host boot up.
               If not specified `false` is assumed.
        :param pulumi.Input[str] bridge: The bridge device defines the name of a bridge
               device which will be used to construct the virtual network (when not provided,
               it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        :param pulumi.Input[pulumi.InputType['NetworkDhcpArgs']] dhcp: DHCP configuration. 
               You need to use it in conjuction with the adresses variable.
        :param pulumi.Input[pulumi.InputType['NetworkDnsArgs']] dns: configuration of DNS specific settings for the network
        :param pulumi.Input[pulumi.InputType['NetworkDnsmasqOptionsArgs']] dnsmasq_options: configuration of Dnsmasq options for the network
               You need to provide a list of option name and value pairs.
        :param pulumi.Input[str] domain: The domain used by the DNS server.
        :param pulumi.Input[str] mode: One of:
               - `none`: the guests can talk to each other and the host OS, but cannot reach
               any other machines on the LAN.
               - `nat`: it is the default network mode. This is a configuration that
               allows guest OS to get outbound connectivity regardless of whether the host
               uses ethernet, wireless, dialup, or VPN networking without requiring any
               specific admin configuration. In the absence of host networking, it at
               least allows guests to talk directly to each other.
               - `route`: this is a variant on the default network which routes traffic from
               the virtual network to the LAN **without applying any NAT**. It requires that
               the IP address range be pre-configured in the routing tables of the router
               on the host network.
               - `open`: similar to `route`, but no firewall rules are added.
               - `bridge`: use a pre-existing host bridge. The guests will effectively be
               directly connected to the physical network (i.e. their IP addresses will
               all be on the subnet of the physical network, and there will be no
               restrictions on inbound or outbound connections). The `bridge` network
               attribute is mandatory in this case.
        :param pulumi.Input[int] mtu: The MTU to set for the underlying network interfaces. When
               not supplied, libvirt will use the default for the interface, usually 1500.
               Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
               Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['NetworkRouteArgs']]]] routes: a list of static routes. A `cidr` and a `gateway` must
               be provided. The `gateway` must be reachable via the bridge interface.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _NetworkState.__new__(_NetworkState)

        __props__.__dict__["addresses"] = addresses
        __props__.__dict__["autostart"] = autostart
        __props__.__dict__["bridge"] = bridge
        __props__.__dict__["dhcp"] = dhcp
        __props__.__dict__["dns"] = dns
        __props__.__dict__["dnsmasq_options"] = dnsmasq_options
        __props__.__dict__["domain"] = domain
        __props__.__dict__["mode"] = mode
        __props__.__dict__["mtu"] = mtu
        __props__.__dict__["name"] = name
        __props__.__dict__["routes"] = routes
        __props__.__dict__["xml"] = xml
        return Network(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def addresses(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        A list of (0 or 1) IPv4 and (0 or 1) IPv6 subnets in
        CIDR notation.  This defines the subnets associated to that network.
        This argument is also used to define the address on the real host.
        If `dhcp {  enabled = true }` addresses is also used to define the address range served by
        the DHCP server.
        No DHCP server will be started if `addresses` is omitted.
        """
        return pulumi.get(self, "addresses")

    @property
    @pulumi.getter
    def autostart(self) -> pulumi.Output[Optional[bool]]:
        """
        Set to `true` to start the network on host boot up.
        If not specified `false` is assumed.
        """
        return pulumi.get(self, "autostart")

    @property
    @pulumi.getter
    def bridge(self) -> pulumi.Output[str]:
        """
        The bridge device defines the name of a bridge
        device which will be used to construct the virtual network (when not provided,
        it will be automatically obtained by libvirt in `none`, `nat`, `route` and `open` modes).
        """
        return pulumi.get(self, "bridge")

    @property
    @pulumi.getter
    def dhcp(self) -> pulumi.Output[Optional['outputs.NetworkDhcp']]:
        """
        DHCP configuration. 
        You need to use it in conjuction with the adresses variable.
        """
        return pulumi.get(self, "dhcp")

    @property
    @pulumi.getter
    def dns(self) -> pulumi.Output[Optional['outputs.NetworkDns']]:
        """
        configuration of DNS specific settings for the network
        """
        return pulumi.get(self, "dns")

    @property
    @pulumi.getter(name="dnsmasqOptions")
    def dnsmasq_options(self) -> pulumi.Output[Optional['outputs.NetworkDnsmasqOptions']]:
        """
        configuration of Dnsmasq options for the network
        You need to provide a list of option name and value pairs.
        """
        return pulumi.get(self, "dnsmasq_options")

    @property
    @pulumi.getter
    def domain(self) -> pulumi.Output[Optional[str]]:
        """
        The domain used by the DNS server.
        """
        return pulumi.get(self, "domain")

    @property
    @pulumi.getter
    def mode(self) -> pulumi.Output[Optional[str]]:
        """
        One of:
        - `none`: the guests can talk to each other and the host OS, but cannot reach
        any other machines on the LAN.
        - `nat`: it is the default network mode. This is a configuration that
        allows guest OS to get outbound connectivity regardless of whether the host
        uses ethernet, wireless, dialup, or VPN networking without requiring any
        specific admin configuration. In the absence of host networking, it at
        least allows guests to talk directly to each other.
        - `route`: this is a variant on the default network which routes traffic from
        the virtual network to the LAN **without applying any NAT**. It requires that
        the IP address range be pre-configured in the routing tables of the router
        on the host network.
        - `open`: similar to `route`, but no firewall rules are added.
        - `bridge`: use a pre-existing host bridge. The guests will effectively be
        directly connected to the physical network (i.e. their IP addresses will
        all be on the subnet of the physical network, and there will be no
        restrictions on inbound or outbound connections). The `bridge` network
        attribute is mandatory in this case.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter
    def mtu(self) -> pulumi.Output[Optional[int]]:
        """
        The MTU to set for the underlying network interfaces. When
        not supplied, libvirt will use the default for the interface, usually 1500.
        Libvirt version 5.1 and greater will advertise this value to nodes via DHCP.
        """
        return pulumi.get(self, "mtu")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        A unique name for the resource, required by libvirt.
        Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def routes(self) -> pulumi.Output[Optional[Sequence['outputs.NetworkRoute']]]:
        """
        a list of static routes. A `cidr` and a `gateway` must
        be provided. The `gateway` must be reachable via the bridge interface.
        """
        return pulumi.get(self, "routes")

    @property
    @pulumi.getter
    def xml(self) -> pulumi.Output[Optional['outputs.NetworkXml']]:
        return pulumi.get(self, "xml")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['IgnitionArgs', 'Ignition']

@pulumi.input_type
class IgnitionArgs:
    def __init__(__self__, *,
                 content: pulumi.Input[str],
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Ignition resource.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
        :param pulumi.Input[str] pool: The pool where the resource will be created.
               If not given, the `default` pool will be used.
        """
        pulumi.set(__self__, "content", content)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)

    @property
    @pulumi.getter
    def content(self) -> pulumi.Input[str]:
        return pulumi.get(self, "content")

    @content.setter
    def content(self, value: pulumi.Input[str]):
        pulumi.set(self, "content", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def pool(self) -> Optional[pulumi.Input[str]]:
        """
        The pool where the resource will be created.
        If not given, the `default` pool will be used.
        """
        return pulumi.get(self, "pool")

    @pool.setter
    def pool(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pool", value)


@pulumi.input_type
class _IgnitionState:
    def __init__(__self__, *,
                 content: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Ignition resources.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
        :param pulumi.Input[str] pool: The pool where the resource will be created.
               If not given, the `default` pool will be used.
        """
        if content is not None:
            pulumi.set(__self__, "content", content)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)

    @property
    @pulumi.getter
    def content(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "content")

    @content.setter
    def content(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "content", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A unique name for the resource, required by libvirt.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def pool(self) -> Optional[pulumi.Input[str]]:
        """
        The pool where the resource will be created.
        If not given, the `default` pool will be used.
        """
        return pulumi.get(self, "pool")

    @pool.setter
    def pool(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pool", value)


class Ignition(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 content: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Create a Ignition resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
        :param pulumi.Input[str] pool: The pool where the resource will be created.
               If not given, the `default` pool will be used.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: IgnitionArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a Ignition resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param IgnitionArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(IgnitionArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 content: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 pool: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = IgnitionArgs.__new__(IgnitionArgs)

            if content is None and not opts.urn:
                raise TypeError("Missing required property 'content'")
            __props__.__dict__["content"] = content
            __props__.__dict__["name"] = name
            __props__.__dict__["pool"] = pool
        super(Ignition, __self__).__init__(
            'libvirt:index/ignition:Ignition',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            content: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            pool: Optional[pulumi.Input[str]] = None) -> 'Ignition':
        """
        Get an existing Ignition resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: A unique name for the resource, required by libvirt.
        :param pulumi.Input[str] pool: The pool where the resource will be created.
               If not given, the `default` pool will be used.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _IgnitionState.__new__(_IgnitionState)

        __props__.__dict__["content"] = content
        __props__.__dict__["name"] = name
        __props__.__dict__["pool"] = pool
        return Ignition(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def content(self) -> pulumi.Output[str]:
        return pulumi.get(self, "content")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        A unique name for the resource, required by libvirt.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def pool(self) -> pulumi.Output[Optional[str]]:
        """
        The pool where the resource will be created.
        If not given, the `default` pool will be used.
        """
        return pulumi.get(self, "pool")

